# Initialisation

> You can skip this chapter if you are not interested in the detail, since all code are written in the class `pycWB` and you can initialise `cWB` with `config/config.ini` and few lines

## Load all required libraries

### References from original bash + C (Won't be used here)
ROOT require a `.root` file to point to the `logon.c`

>  `.root`: ${SOURCE}/tools/cwb/cwb.rootrc
 
```bash
 #Browser.Name:            TRootBrowser
Browser.Name:            TRootBrowserLite
#Rint.Logon:              rootlogon.C
Rint.Logon:              $CWB_ROOTLOGON_FILE
```

> `logon.c`: ${INSTALL}/etc/cwb/macros/cwb_rootlogon.C

```bash
_USE_ROOT6 = 
_USE_ICC = 
_USE_HEALPIX = 
HOME_CFITSIO = # leave empty if installed with conda
HOME_HEALPIX = # leave empty if installed with conda

_USE_LAL = 
HOME_LAL = # leave empty if installed with conda

_USE_EBBH = # optional
HOME_CVODE = 

HOME_WAT = 
HOME_WAT_INSTALL = 

HOME_FRLIB = 
CWB_STFT = 
CWB_GWAT = 
CWB_TOOLBOX = 
CWB_HISTORY = 
CWB_BICO = 
CWB_FRAME = 
CWB_FILTER = 
HOME_CWB = 
CWB_MACROS = 
```

### Python code replacement
Remove `.so` for all relative imports in `cwb_rootlogon` to enable import both `.so` and `.dylib`. ROOT can automatically find the library file with various extension.

You can test the following codes in `ipython`

```python
_USE_ROOT6 = True
_USE_HEALPIX = True
_USE_LAL = True

HOME_CFITSIO = "" # leave empty if installed with conda
HOME_HEALPIX = "" # leave empty if installed with conda
HOME_LAL = "" # leave empty if installed with conda

_USE_ICC = False # optional

_USE_EBBH = False # optional
HOME_CVODE = ""

# HOME_WAT = "~/Project/Physics/cwb/cwb_source"
# HOME_WAT_INSTALL = "~/Project/Physics/cwb/cwb_source/tools/install"
CWB_INSTALL = "/Users/yumengxu/Project/Physics/cwb/cwb_source/tools/install" # CAUTION: Do not use relative path here!
CWB_SOURCE = "/Users/yumengxu/Project/Physics/cwb/cwb_source" # For loading macros
HOME_FRLIB = "" # leave empty if installed with conda
# CWB_STFT = 
# CWB_GWAT = 
# CWB_TOOLBOX = 
# CWB_HISTORY = 
# CWB_BICO = 
# CWB_FRAME = 
# CWB_FILTER = 
HOME_CWB = ""
CWB_MACROS = ""

import platform
import os

os.environ['LD_LIBRARY_PATH'] = f"{CWB_INSTALL}/lib" # TODO: add previous LD path

from ROOT import gROOT, gSystem, gStyle
import ROOT

# include paths
include_path = [f"{CWB_INSTALL}/include"]
if _USE_EBBH: include_path += [f"{HOME_CVODE}/include"]

for inc in include_path:
	# TODO: error process
	gROOT.ProcessLine(f".include {inc}")

# load libraries
root_lib = ['libPhysics', 'libFFTW', 'libHtml', 'libTreeViewer', 'libpng', 'libFITSIO'] #libFITSIO mac only?
lal_lib = ['liblal', 'liblalsupport', 'liblalframe', 'liblalmetaio', 'liblalsimulation', 'liblalinspiral', 'liblalburst']
healpix_lib = ['libhealpix_cxx', 'libgomp']
wat_lib = ['wavelet']
ebbh = ['eBBH']# TODO

libs = root_lib
if _USE_LAL: libs += lal_lib
if _USE_HEALPIX: libs += healpix_lib
libs += wat_lib
if _USE_EBBH: libs += ebbh

for lib in libs:
	# TODO: error process and version check (healpix >= 3.00)
	# use user path if can't find lal healpix etc. in default
	gSystem.Load(lib)

# Loading Macros
wat_macros = ["Histogram.C", "AddPulse.C", "readAscii.C", "readtxt.C"]

for macro in wat_macros:
	gROOT.LoadMacro(f"{CWB_SOURCE}/wat/macro/{macro}")

# Loading tools
tools_lib = ['STFT', 'gwat', 'Toolbox', 'History', 'Bicoherence', 'Filter', 'frame', 'cwb', 'wavegraph']

for lib in tools_lib:
	# TODO: error process
	gSystem.Load(f"{CWB_INSTALL}/lib/{lib}")


# declare ACLiC includes environment 
for inc in include_path:
	gSystem.AddIncludePath(f"-I{inc}")

# declare ACLiCFlag options 
flag = "-D_USE_ROOT -fPIC -Wno-deprecated -mavx -Wall -Wno-unknown-pragmas -fexceptions -O2 -D__STDC_CONSTANT_MACROS"
if _USE_HEALPIX: flag += " -D_USE_HEALPIX"
if _USE_LAL: flag += " -D_USE_LAL"
if _USE_EBBH: flag += " -D_USE_EBBH"
if _USE_ROOT6: flag += " -D_USE_ROOT6"
if platform.system() == "Darwin": flag += " -fno-common -dynamiclib -undefined dynamic_lookup"
flag += " -fopenmp"
if _USE_ICC: flag += " -diag-disable=2196"


gSystem.SetFlagsOpt(flag);

# set the offset for TimeDisplay, the seconds declared in xaxis
# are refered to "1980-01-06 00:00:00 UTC Sun" -> GPS = 0
gStyle.SetTimeOffset(315964790); 

gStyle.SetPalette(1,0);
gStyle.SetNumberContours(256);

gROOT.ForceStyle(0);
```


If some error happens, try restart first.

## Setup the environment and initialise the parameters

Convert `watenv.sh`. Below should be replaced by yaml file in the future

```python
envs = {
	"HOME_WAT_FILTERS": "path",
	"HOME_BAUDLINE": "baudline", # independent, path of software
	"HOME_ALADIN": "aladin", # independent, path of software
	"HOME_SKYMAP_LIB": "path", # required
	"CWB_USER_URL": "url", # report url
	"WWW_PUBLIC_DIR": "~/Downloads/tmp/public_html/reports",
	"CONDOR_LOG_DIR": "~/Downloads/tmp/condor",
	"NODE_DATA_DIR": "~/Downloads/tmp/node",
	"CWB_ANALYSIS": '2G',
	"CWB_PARAMETERS_FILE": f"{CWB_INSTALL}/etc/cwb/macros/cwb2G_parameters.C",
	"CWB_ROOTLOGON_FILE": "",
	"CWB_UPARAMETERS_FILE": "",
	"CWB_EPARAMETERS_FILE": "",
	}

envs['CWB_PARMS_FILES'] = f"{envs['CWB_ROOTLOGON_FILE']} {envs['CWB_PARAMETERS_FILE']} {envs['CWB_UPARAMETERS_FILE']} {envs['CWB_EPARAMETERS_FILE']}"

for key in envs.keys():
	os.environ[key] = envs[key]

gROOT.LoadMacro(envs['CWB_PARMS_FILES'])
```

> Below error will occur if the path is relative
> ```
> Error in <TROOT::Macro>: macro $CWB_PARAMETERS_FILE not found in path .:/usr/local/anaconda3/envs/cwb/macros
> ```
