# Migrate `cwb_inet2G` to python

```python
from pycwb import pycWB, sim

cwb = pycWB('./config.ini')  # config file path
ROOT = cwb.ROOT
gROOT = cwb.gROOT

# create frame file

sim.create_frame_noise(gROOT, ROOT)
sim.setup_sim_data(['H1','L1','V1'])

# run full `cwb_inet2G` analysis

job_id = 1
job_stage = 'FULL'
job_file = './user_parameters.yaml'
inet_option = '--tool emax --level 8  --draw true'
cwb.cwb_inet2G(job_id, job_file, job_stage, inet_option=inet_option)
```

`c++` code to convert

```c
cwb2G* CWB = NULL;

if(eced) { 				// get fName from env option --cfg
    TString cwb_eced_opts=TString(gSystem->Getenv("CWB_ECED_OPTS"));
    if(cwb_eced_opts!="") {
      TString ECED_CFG = CWB::Toolbox::getParameter(cwb_eced_opts,"--cfg");
      if(ECED_CFG!="") {
        fName = gSystem->ExpandPathName(ECED_CFG.Data());
      }
    }
}

CWB::config icfg;
icfg.Import("$CWB_PARAMETERS_FILE");

if(eced) { 				// skip config::check (used by eced) 
  int nIFO=1;int nfactor=1;   	
  EXPORT(int,nIFO,TString::Format("nIFO = %d",nIFO).Data())
  EXPORT(int,nfactor,TString::Format("nfactor = %d",nfactor).Data())
}   

if(eced) {nIFO=1;nfactor=1;}

icfg.Import(fName);
strcpy(icfg.analysis,"2G");
icfg.Export();
CWB = new cwb2G(icfg,jstage);
// updated standard user config (saved in CED)
gSystem->Setenv("CWB_UPARAMETERS_FILE",fName);


CWB::config* cfg = CWB->GetConfig();
TArrayC lagBuffer;  
if(CWB->GetLagBuffer().GetSize()) {  
  char* lagFile=NULL;
  char  lagMode[2];
  lagBuffer = CWB->GetLagBuffer();
  lagFile = lagBuffer.GetArray();
  lagMode[0] = CWB->GetLagMode();
  EXPORT(char*,lagFile,TString::Format("lagFile = (char*)%p",lagFile).Data())
  EXPORT(char*,lagMode,TString::Format("strcpy(lagMode,(char*)%p)",lagMode).Data())
  strcpy(cfg->lagMode,lagMode);
  cfg->Import();
}
if(eced) cfg->Import(gSystem->ExpandPathName("$CWB_MACROS/cwb_eced.C"));  // easy ced
cfg->Import(gSystem->ExpandPathName("$CWB_MACROS/cwb_inet.C"));
CWB->SetupStage(jstage);
char tmp_dir[1024]; strcpy(tmp_dir,cfg->tmp_dir);
if(eced) cfg->Print(TString(tmp_dir)+"/eced_parameters.C");  // dump full parameters file
CWB->run(runID);
delete CWB;
```

`ipython` test code

```python
from pycwb import pycWB

cwb = pycWB('./config.ini')  # config file path
ROOT = cwb.ROOT
gROOT = cwb.gROOT

cwb.user_params_with_yaml('./user_parameters.yaml')

icfg = ROOT.CWB.config()
icfg.Import()

CWB = ROOT.cwb2G(icfg, 1)

cfg = CWB.GetConfig()
# read lags from job file (used in multistage analysis)
lagBuffer = CWB.GetLagBuffer()
lagFile = lagBuffer.GetArray()
lagMode = CWB.GetLagMode()
# cfg.lagFile = lagFile
cfg.lagMode = lagMode
cfg.Export()
```